searchNodes=[{"doc":"A Trunk is a module used to handle the transformation and storage of files with multiple versions. Usage: To generate a Trunk call use Trunk and supply options to configure the specific behaviour you need. defmodule MyTrunk do use Trunk , versions : [ :original ] , storage : Trunk.Storage.Filesystem , storage_opts : [ path : &quot;/tmp&quot; ] end Options Options can be set at multiple levels Global, use config :trunk, key: :value Per otp app, use config :my_app, trunk: [key: :value] Per module, supply options to use Trunk . Use use Trunk, otp_app: :my_app to read in per otp app options. Per function, you can supply additional or override options to every function call. Available options :versions a list of versions as atoms, [:original, :thumb] :async (boolean) default true , whether to process each version in parallel or in sequence. :timeout default 5_000 , how long to wait for each versions transformation and storage to complete when processing in parallel. Note: if storing files non-locally (i.e. Amazon S3), then include both expected time for transformation and expected time for storage in this timeout. :storage default Trunk.Storage.Filesystem , the storage module to use when storing files and versions :storage_opts default: [path: &quot;&quot;] , the options for the storage module. See each storage module's documentation for available options. :otp_app , only used at module level to read options specific to the otp app. Additional options Any extra options passed in at the otp app, module or function level are made available in the opts field of Trunk.State . This means that you can pass in any options you might need to use in any of the processing steps. File storage When files are stored, they are passed through a transformation pipeline which allows you to generate different versions of a file. You might get a photo uploaded which you want to save along with a thumbnail. You might have a video which also needs a thumbnail extracted. You could convert a spreadsheet to CSV or a document to PDF. At each point in the pipeline you have access to a Trunk.State or Trunk.VersionState struct which contains information collected at each step. Each state struct has an assigns map (similar to Plug.Conn ) in which you can store information to be used later in the pipeline process or once storage is complete. The storage pipeline goes through the following steps: preprocess/1 - Here you have access to the information about the file including the path to the file on disk. This is where you can validate the file or extract information that can be useful for later processing. This callback is called only once for the pipeline. transform/2 - This is where you determine how each version should be transformed. You can return a transformation instruction or a function that will do the actual transformation. This callback is called once per version. postprocess/3 - Here you have access to information about the version after its transformation. At this point you have access to the state about the version including the path to the temporary file on disk. You can extract information about the version file like file size or hash its contents. This callback is called once per version. storage_dir/2 - This is where you determine at which path the version should be saved. This callback is called once per version. filename/2 - This is where you determine which filename the version should be saved as. This callback is called once per version. storage_opts/2 - This allows you to set additional storage options like file permissions :acl or add things like S3 headers (Check with the storage module as to what options are avialble to be set here). This callback is called once per version. The file is saved using the configured storage module. Example: defmodule MyTrunk do use Trunk , versions : [ :original , :thumb ] , storage : Trunk.Storage.Filesystem , storage_opts : [ path : &quot;/tmp&quot; ] # Ensure the file is a JPG or PNG file def preprocess ( % Trunk.State { } = state ) do if String . downcase ( extname ) in [ &quot;.png&quot; , &quot;.jpg&quot; , &quot;.jpeg&quot; ] do { :ok , state } else { :error , &quot;Invalid file&quot; } end end # Do not transform the original file def transform ( % Trunk.State { } , :original ) , do : nil # Resize the file to a thumbnail of maximum 200x200px def transform ( % Trunk.State { } , :thumb ) , do : { :convert , &quot;-strip -thumbnail 200x200&gt; -limit area 10MB -limit disk 100MB&quot; } # Store the file size of each version def postprocess ( % Trunk.VersionState { temp_path : temp_path } = version_state , _version , % Trunk.State { } = _state ) do % File.State { size : file_size } = File . stat ( temp_path ) { :ok , Trunk.VersionState . assign ( version_state , :file_size , file_size ) } end # Store all versions in a directory based on a model id def storage_dir ( % Trunk.State { scope : %{ id : model_id } } , _version ) , do : to_string ( model_id ) # Store the original file with its original filename def filename ( % Trunk.State { filename : filename } , :original ) , do : filename # Store other versions with the version in its filename def filename ( % Trunk.State { rootname : rootname , extname : extname } , version ) , do : &quot; \#{ rootname } _ \#{ version } \#{ extname } &quot; def storage_opts ( % Trunk.State { } , :original ) , do : [ acl : 0o600 ] def storage_opts ( % Trunk.State { } , :thumb ) , do : [ acl : 0o644 ] end &gt; { :ok , % Trunk.State { filename : filename , versions : versions } } = MyTrunk . store ( &quot;/path/to/photo.jpg&quot; ) &gt; filename &quot;photo.jpg&quot; &gt; versions |&gt; Enum . map ( fn ( { version , % Trunk.VersionState { assigns : %{ file_size : file_size } } } ) -&gt; { version , file_size } end ) [ original : 34567 , thumb : 456 ]","ref":"Trunk.html","title":"Trunk","type":"behaviour"},{"doc":"Copies the stored file(s) from the storage_dir/2 and filename/2 of the original scope to the same of the new scope. This function is generated by use Trunk It returns an {:ok, %Trunk.State{}} tuple of the new state on success and {:error, %Trunk.State{}} tuple of the new state if an error occurred anywhere in the processing pipeline. file_info - The file name or base info needed to identify the file. Minimum is %{filename: &quot;file.ext&quot;} scope - a map or struct that will help when generating the filename and storage directory for retrieving the file(s) new_scope - a map or struct that will help when generating the filename and storage directory for saving the file(s) opts - (optional) options to override module, app, or global options. See &quot;Options&quot; in the module documentations for all options.","ref":"Trunk.html#c:copy/4","title":"Trunk.copy/4","type":"callback"},{"doc":"Deletes the identified file and all its versions. This function is generated by use Trunk Calls delete/3 with delete(file_info, nil, [])","ref":"Trunk.html#c:delete/1","title":"Trunk.delete/1","type":"callback"},{"doc":"Deletes the identified file and all its versions. This function is generated by use Trunk With a scope , calls delete/3 with delete(file_info, scope, []) With opts , calls delete/3 with delete(file_info, nil, opts)","ref":"Trunk.html#c:delete/2","title":"Trunk.delete/2","type":"callback"},{"doc":"Deletes the identified file and all its versions. This function is generated by use Trunk It returns an {:ok, %Trunk.State{}} tuple on success and {:error, %Trunk.State{}} tuple if an error occurred. file_info - The file name or base info needed to identify the file. Minimum is %{filename: &quot;file.ext&quot;} scope - (optional) a map or struct that will help when generating the filename and storage directory for saving the file opts - (optional) options to override module, app, or global options. See &quot;Options&quot; in the module documentations for all options.","ref":"Trunk.html#c:delete/3","title":"Trunk.delete/3","type":"callback"},{"doc":"A callback that should be used to generate the filename specific to each version. Return nil to prevent storing a specific version state - The trunk state version - An atom representing the version Example: # For the :original version return the original file name def filename ( %{ filename : filename } , :original ) , do : filename # For any other version append the version to the root name # mypic.jpg with :thumb version becomes mypic_thumb.jpg def filename ( %{ rootname : rootname , extname : extname } , version ) , do : &quot; \#{ rootname } _ \#{ version } \#{ extname } &quot;","ref":"Trunk.html#c:filename/2","title":"Trunk.filename/2","type":"callback"},{"doc":"A callback that can be used to do additional processing on each version file before it gets saved. This callback can update the version state information which can then be used in storage_dir/2 and filename/2 as additional information for storage. Note: If information is stored in the version state and used for file naming, then that information needs to be stored in order to retrieve that file or generate a URL etc. Example: To generate and store the md5 hash of each version. def postprocess ( % Trunk.VersionState { temp_path : temp_path } = version_state , _version , _state ) do hash = :crypto . hash ( :md5 , File . read! ( temp_path ) ) |&gt; Base . encode16 ( case : :lower ) { :ok , Trunk.VersionState . assign ( version_state , :hash , hash ) } end Example: To calculate and store the file size of each version. def postprocess ( % Trunk.VersionState { temp_path : temp_path } = version_state , _version , _state ) do { :ok , % File.Stat { size : file_size } } = File . stat ( temp_path ) { :ok , Trunk.VersionState . assign ( version_state , :file_size , file_size ) } end","ref":"Trunk.html#c:postprocess/3","title":"Trunk.postprocess/3","type":"callback"},{"doc":"A callback that can be used to do any preprocessing on the state before transformation and storage begins. This is a good place to do file validation ( Note this example is wrapped in a convenience macro validate_file_extensions/1 . Example def preprocess ( % Trunk.State { lower_extname : extname } = state ) do if extname in [ &quot;.jpg&quot; , &quot;.png&quot; ] do { :ok , state } else { :error , &quot;Invalid file&quot; } end end This is also a place to do any processing that might be needed when transforming versions.","ref":"Trunk.html#c:preprocess/1","title":"Trunk.preprocess/1","type":"callback"},{"doc":"Reprocesses the original file for all versions. This function is generated by use Trunk Calls reprocess/4 with reprocess(file_info, nil, :all, [])","ref":"Trunk.html#c:reprocess/1","title":"Trunk.reprocess/1","type":"callback"},{"doc":"Reprocesses the original file for all or the specified version(s). This function is generated by use Trunk With a scope , calls reprocess/4 with reprocess(file_info, scope, :all, []) With opts , calls reprocess/4 with reprocess(file_info, nil, :all, opts) With version , calls reprocess/4 with reprocess(file_info, nil, version, [])","ref":"Trunk.html#c:reprocess/2","title":"Trunk.reprocess/2","type":"callback"},{"doc":"Reprocesses the original file for all or the specified version(s). This function is generated by use Trunk With version(s) and opts, calls reprocess/4 with reprocess(file_info, nil, version, opts) With a scope and version(s), calls reprocess/4 with reprocess(file_info, scope, version, []) With a scope and opts, calls reprocess/4 with reprocess(file_info, scope, :all, opts)","ref":"Trunk.html#c:reprocess/3","title":"Trunk.reprocess/3","type":"callback"},{"doc":"Reprocesses the original file for the specified version(s). This function is generated by use Trunk It returns an {:ok, path} tuple on success and {:error, error} tuple if an error occurred. file_info - The file name or base info needed to identify the file. Minimum is %{filename: &quot;file.ext&quot;} scope - (optional) a map or struct that will help when generating the filename and storage directory for saving the file version - (optional) an atom representing the version opts - (optional) options to override module, app, or global options. See &quot;Options&quot; in the module documentations for all options.","ref":"Trunk.html#c:reprocess/4","title":"Trunk.reprocess/4","type":"callback"},{"doc":"Retrieves the original for the identified file. This function is generated by use Trunk Calls retrieve/4 with retrieve(file_info, nil, :original, [])","ref":"Trunk.html#c:retrieve/1","title":"Trunk.retrieve/1","type":"callback"},{"doc":"Retrieves the original, or specified version, for the identified file. This function is generated by use Trunk With a scope , calls retrieve/4 with retrieve(file_info, scope, :original, []) With opts , calls retrieve/4 with retrieve(file_info, nil, :original, opts) With version , calls retrieve/4 with retrieve(file_info, nil, version, [])","ref":"Trunk.html#c:retrieve/2","title":"Trunk.retrieve/2","type":"callback"},{"doc":"Retrieves the original, or specified version, for the identified file. This function is generated by use Trunk With a version and opts, calls retrieve/4 with retrieve(file_info, nil, version, opts) With a scope and version, calls retrieve/4 with retrieve(file_info, scope, version, []) With a scope and opts, calls retrieve/4 with retrieve(file_info, scope, :original, opts)","ref":"Trunk.html#c:retrieve/3","title":"Trunk.retrieve/3","type":"callback"},{"doc":"Retrieves the version for the identified file. This function is generated by use Trunk It returns an {:ok, path} tuple on success and {:error, error} tuple if an error occurred. file_info - The file name or base info needed to identify the file. Minimum is %{filename: &quot;file.ext&quot;} scope - (optional) a map or struct that will help when generating the filename and storage directory for saving the file version - (optional) an atom representing the version opts - (optional) options to override module, app, or global options. See &quot;Options&quot; in the module documentations for all options. Additional options : - ` :output_path ` a specific path ( including filename ) to save the file to . Defaults to a temporary file path","ref":"Trunk.html#c:retrieve/4","title":"Trunk.retrieve/4","type":"callback"},{"doc":"A callback that should be used to determine the storage directory specific to each version. Each storage system will have a base path in which to store files. The result of storage_dir/2 is then appended to this to determine where to save files. state - The trunk state version - An atom representing the version Example: # Assuming a base path of /tmp files will be saved in /tmp def storage_dir ( _state , _version ) , do : &quot;&quot; # Place the file in a directory named after a model id # With a base path of /tmp and a scope of %{id: 42}, the file will be saved in /tmp/42/ def storage_dir ( % Trunk.State { scope : %{ id : model_id } } , _version ) , do : to_string ( model_id )","ref":"Trunk.html#c:storage_dir/2","title":"Trunk.storage_dir/2","type":"callback"},{"doc":"A callback that should be used to set version specific storage options. This is the place to set access permissions, storage headers (s3) and other options provided by the storage module. This callback should return a list of options. Example: # With Filesystem def storage_opts ( _state , _version ) , do : [ acl : &quot;0600&quot; ] # With S3 def storage_opts ( _state , _version ) , do : [ acl : :public_read ]","ref":"Trunk.html#c:storage_opts/2","title":"Trunk.storage_opts/2","type":"callback"},{"doc":"Stores the supplied file. This function is generated by use Trunk Calls store/3 with store(file, nil, [])","ref":"Trunk.html#c:store/1","title":"Trunk.store/1","type":"callback"},{"doc":"Stores the supplied file. This function is generated by use Trunk Calls store/3 with store(file, nil, opts)","ref":"Trunk.html#c:store/2","title":"Trunk.store/2","type":"callback"},{"doc":"Stores the supplied file after running it through the processing pipeline. This function is generated by use Trunk It returns an {:ok, %Trunk.State{}} tuple on success and {:error, %Trunk.State{}} tuple if an error occurred anywhere in the processing pipeline. file - the file to store. This can be a full path to file, or a url to a file, or a map with :filename and :path keys (e.g. %Plug.Upload{} ), or a map with :filename and :binary keys (e.g. %{filename: &quot;myfile.jpg&quot;, binary: &lt;&lt;…&gt;&gt;} ) scope - (optional) a map or struct that will help when generating the filename and storage directory for saving the file opts - (optional) options to override module, app, or global options. See &quot;Options&quot; in the module documentations for all options.","ref":"Trunk.html#c:store/3","title":"Trunk.store/3","type":"callback"},{"doc":"A callback that can be used to generate a transform specific to each version The transformation instruction is a two element tuple {command, arguments} , a three element tuple {command, arguments, extension} , or a function that accepts a single argument (the source file path) and returns either {:ok, &quot;/path/to/transformed/file&quot;} or {:error, &quot;Reason&quot;} Example: To generate a thumnail, you might have a version named :thumb . A transform that runs the uploaded file through ImageMagick’s convert could be the following def transform ( % Trunk.State { } , :thumb ) , do : { :convert , &quot;-strip -thumbnail 200x200&gt;&quot; } This would generate a call to System.cmd as follows: # input_file is the source (uploaded) file # output_file is a temporary file with the same extension as the source file System . cmd &quot;convert&quot; , [ input_file , &quot;-strip&quot; , &quot;-thumbnail&quot; , &quot;200x200&gt;&quot; , output_file ] , stderr_to_stdout : true To generate a JPEG thumbnail whether the source is JPEG or PNG you would add the extension to the transform instruction. def transform ( _state , :thumb ) , do : { :convert , &quot;-strip -thumbnail 200x200&gt;&quot; , :jpg } In this case the temporary file generated for the transformation would have a .jpg extension. In either form, the arguments can be generated by a function which receives the input and output file paths. This is useful for transforms where the input and output paths are not the first and last respectively. def transform ( _state , _version ) , do : { :convert , fn ( input , output ) -&gt; [ &quot;-density&quot; , &quot;300&quot; , input , &quot;-flatten&quot; , &quot;-strip&quot; , &quot;-thumbnail&quot; , &quot;200x200&gt;&quot; , output ] end , :jpg } Note: Returning an instruction with an extension will change the extension of the temporary file used in the transformation but will not affect the filename during save. That still needs to be done in filename/2 Finally a function can be returned which will be wholly responsible for doing the transformation and returning the location of the transformed file. The function should return {:ok, file_path} if successful, or {:error, reason} if not. This example uses soffice to transform an XLS document to CSV. The problem is soffice doesn’t allow the output file to be specified so the above {:soffice, [args]} style can’t be used. To solve this, a transformation function is used to handle the transformation. The function creates a temporary directory for the transformation, runs the soffice command and then parses the output for the resulting file. def transform ( % Trunk.State { extname : extname } , :csv ) when extname in [ &quot;.xls&quot; , &quot;.xlsx&quot; ] , do : fn ( source_path ) -&gt; { :ok , directory } = Briefly . create ( directory : true ) # soffice must be in the PATH case System . cmd ( &quot;soffice&quot; , [ &quot;--headless&quot; , &quot;--convert-to&quot; , &quot;csv:Text - txt - csv (StarCalc):44,34,76,0&quot; , &quot;--outdir&quot; , directory , source_path ] , stderr_to_stdout : true ) do { output , 0 } -&gt; [ _ , path ] = Regex . run ( ~r/-&gt; (.+ \\. csv)/ , output ) { :ok , path } { result , _ } -&gt; { :error , result } end end def transform ( _state , _version ) , do : nil","ref":"Trunk.html#c:transform/2","title":"Trunk.transform/2","type":"callback"},{"doc":"Generates a URL for the given information. This function is generated by use Trunk Will return nil if the filename/2 callback returns nil Calls url/4 with url(file_info, nil, :original, [])","ref":"Trunk.html#c:url/1","title":"Trunk.url/1","type":"callback"},{"doc":"Generates a URL for the given information. This function is generated by use Trunk Will return nil if the filename/2 callback returns nil With a version , calls url/4 with url(file_info, nil, version, []) With a scope , calls url/4 with url(file_info, scope, :original, []) With opts , calls url/4 with url(file_info, nil, :original, opts)","ref":"Trunk.html#c:url/2","title":"Trunk.url/2","type":"callback"},{"doc":"Generates a URL for the given information. This function is generated by use Trunk Will return nil if the filename/2 callback returns nil With a version , calls url/4 with url(file_info, nil, version, opts) With a scope , calls url/4 with url(file_info, scope, :original, opts)","ref":"Trunk.html#c:url/3","title":"Trunk.url/3","type":"callback"},{"doc":"Generates a URL for the given information. This function is generated by use Trunk Will return nil if the filename/2 callback returns nil file_info - The file name or base info needed to identify the file. Minimum is %{filename: &quot;file.ext&quot;} scope - The scope object version - The file version to which the URL must point opts - Override options.","ref":"Trunk.html#c:url/4","title":"Trunk.url/4","type":"callback"},{"doc":"A convenience macro for generating a preprocess/1 function that validates the file being saved against a list of approved extensions (case-insensitive). Example: defmodule MyTrunk do use Trunk , versions : [ :original ] validate_file_extensions ~w[.jpg .jpeg .png] end","ref":"Trunk.html#validate_file_extensions/1","title":"Trunk.validate_file_extensions/1","type":"macro"},{"doc":"","ref":"Trunk.html#t:args/0","title":"Trunk.args/0","type":"type"},{"doc":"","ref":"Trunk.html#t:command/0","title":"Trunk.command/0","type":"type"},{"doc":"","ref":"Trunk.html#t:ext/0","title":"Trunk.ext/0","type":"type"},{"doc":"","ref":"Trunk.html#t:file/0","title":"Trunk.file/0","type":"type"},{"doc":"","ref":"Trunk.html#t:file_info/0","title":"Trunk.file_info/0","type":"type"},{"doc":"","ref":"Trunk.html#t:new_scope/0","title":"Trunk.new_scope/0","type":"type"},{"doc":"","ref":"Trunk.html#t:opts/0","title":"Trunk.opts/0","type":"type"},{"doc":"","ref":"Trunk.html#t:reason/0","title":"Trunk.reason/0","type":"type"},{"doc":"","ref":"Trunk.html#t:scope/0","title":"Trunk.scope/0","type":"type"},{"doc":"","ref":"Trunk.html#t:storage_opts/0","title":"Trunk.storage_opts/0","type":"type"},{"doc":"","ref":"Trunk.html#t:transform_func/0","title":"Trunk.transform_func/0","type":"type"},{"doc":"","ref":"Trunk.html#t:version/0","title":"Trunk.version/0","type":"type"},{"doc":"","ref":"Trunk.html#t:versions/0","title":"Trunk.versions/0","type":"type"},{"doc":"This module defines a Trunk.State struct and provides some helper functions for working with that state. Fields The following fields are available in the state object. Some values are filled in during processing. filename - The base filename of the file being processed. (e.g. &quot;photo.jpg&quot; ) rootname - The root of the filen being processed. (e.g. &quot;photo&quot; ) extname - The file extension of the file being processed. (e.g. &quot;.jpg&quot; ) lower_extname - The file extension of the file being processed forced to lower case (e.g. &quot;*.jpg&quot; , even if the file is &quot;PHOTO.JPG&quot; ) path - The full path to the file being processed. If the file was passed in as a binary, it is a path to the temporary file created with that binary. versions - A map of the versions and their respective Trunk.VersionState (e.g. %{original: %Trunk.VersionState{}, thumbnail: %Trunk.VersionState{}} ) scope - A user struct/map passed in useful for determining storage locations and file naming. async - A boolean indicator of whether processing will be done in parallel. timeout - The timeout after which each processing process will be terminated. (Only applies with async: true ) storage - The module to use for storage processing. (e.g. Trunk.Storage.Filesystem or Trunk.Storage.S3 ) storage_opts - A keyword list of options for the storage module errors - a place to record errors encountered during processing. ( nli if no errors, otherwise a map of errors) opts - All the options merged together (see Options in Trunk module documentation). assigns - shared user data as a map (Same as assigns in Plug.Conn )","ref":"Trunk.State.html","title":"Trunk.State","type":"module"},{"doc":"Assigns a value to a key on the state. Example: iex&gt; state . assigns [ :hello ] nil iex&gt; state = Trunk.State . assign ( state , :hello , :world ) iex&gt; state . assigns [ :hello ] :world","ref":"Trunk.State.html#assign/3","title":"Trunk.State.assign/3","type":"function"},{"doc":"Retrieves an assign value for a specific version. Example: iex&gt; state = % Trunk.State { versions : %{ thumbnail : % Trunk.VersionState { assigns : %{ hello : :world } } } } iex&gt; % Trunk.State . get_version_assign ( state , :thumbnail , :hello ) :world iex&gt; % Trunk.State . get_version_assign ( state , :thumbnail , :unknown ) nil","ref":"Trunk.State.html#get_version_assign/3","title":"Trunk.State.get_version_assign/3","type":"function"},{"doc":"","ref":"Trunk.State.html#init/3","title":"Trunk.State.init/3","type":"function"},{"doc":"Puts an error into the error map. Example: iex&gt; state . errors nil iex&gt; state = Trunk.State . put_error ( state , :thumb , :transform , &quot;Error with convert blah blah&quot; ) iex&gt; state . errors %{ thumb : [ transform : &quot;Error with convert blah blah&quot; ] }","ref":"Trunk.State.html#put_error/4","title":"Trunk.State.put_error/4","type":"function"},{"doc":"Restore a saved state from a filename, JSON, or a map Example: iex&gt; Trunk.State . restore ( &quot;photo.jpg&quot; ) % Trunk.State { filename : &quot;photo.jpg&quot; } iex&gt; Trunk.State . restore ( %{ filename : &quot;photo.jpg&quot; , assigns : %{ hash : &quot;abcdef&quot; } } % Trunk.State { filename : &quot;photo.jpg&quot; , assigns : %{ hash : &quot;abcdef&quot; } } iex&gt; Trunk.State . restore ( %{ &quot;filename&quot; =&gt; &quot;photo.jpg&quot; , &quot;assigns&quot; =&gt; %{ &quot;hash&quot; =&gt; &quot;abcdef&quot; } } % Trunk.State { filename : &quot;photo.jpg&quot; , assigns : %{ hash : &quot;abcdef&quot; } } iex&gt; Trunk.State . restore ( &quot;{ \\&quot; filename \\&quot; : \\&quot; photo.jpg \\&quot; , \\&quot; assigns \\&quot; : { \\&quot; hash \\&quot; : \\&quot; abcdef \\&quot; }}&quot; ) % Trunk.State { filename : &quot;photo.jpg&quot; , assigns : %{ hash : &quot;abcdef&quot; } }","ref":"Trunk.State.html#restore/2","title":"Trunk.State.restore/2","type":"function"},{"doc":"Extracts the data needed from the state in order to reconstruct the file paths in future. Options: :as - How to save the state. :string - Default, will just save the file name. An error will be raised if there are any assigns unless :ignore_assigns is set to tru :map - will save a map with keys :filename , :assigns , and :version_assigns :json - will save a map encoded as JSON (Requires Poison library to be included in deps) :ignore_assigns boolean, default false. Use this to save as string and ignore any assigns (Make sure you’re not using assigns for Trunk.storage_dir/2 or Trunk.filename/2 ) :assigns - a list of keys to save from the assigns hashes Example: iex&gt; Trunk.State . save ( % Trunk.State { filename : &quot;photo.jpg&quot; } ) &quot;photo.jpg&quot; iex&gt; Trunk.State . save ( % Trunk.State { filename : &quot;photo.jpg&quot; , assigns : %{ hash : &quot;abcdef&quot; } } , as : :map ) %{ filename : &quot;photo.jpg&quot; , assigns : %{ hash : &quot;abcdef&quot; } } iex&gt; Trunk.State . save ( % Trunk.State { filename : &quot;photo.jpg&quot; , assigns : %{ hash : &quot;abcdef&quot; , file_size : 12345 } } , as : :map , assigns : [ :hash ] ) %{ filename : &quot;photo.jpg&quot; , assigns : %{ hash : &quot;abcdef&quot; } } iex&gt; Trunk.State . save ( % Trunk.State { filename : &quot;photo.jpg&quot; , assigns : %{ hash : &quot;abcdef&quot; } } , as : :json ) &quot;{ \\&quot; filename \\&quot; : \\&quot; photo.jpg \\&quot; , \\&quot; assigns \\&quot; : { \\&quot; hash \\&quot; : \\&quot; abcdef \\&quot; }}&quot;","ref":"Trunk.State.html#save/2","title":"Trunk.State.save/2","type":"function"},{"doc":"","ref":"Trunk.State.html#t:assign_keys/0","title":"Trunk.State.assign_keys/0","type":"type"},{"doc":"","ref":"Trunk.State.html#t:file_info/0","title":"Trunk.State.file_info/0","type":"type"},{"doc":"","ref":"Trunk.State.html#t:opts/0","title":"Trunk.State.opts/0","type":"type"},{"doc":"","ref":"Trunk.State.html#t:save_opts/0","title":"Trunk.State.save_opts/0","type":"type"},{"doc":"","ref":"Trunk.State.html#t:t/0","title":"Trunk.State.t/0","type":"type"},{"doc":"","ref":"Trunk.State.html#t:version/0","title":"Trunk.State.version/0","type":"type"},{"doc":"This is a behaviour that can be implemented by any storage module to make sure it supports all required functions.","ref":"Trunk.Storage.html","title":"Trunk.Storage","type":"behaviour"},{"doc":"Generates a URL to the S3 object directory - The directory filename - The name of the file opts - The options for the storage system, set in Trunk option :storage_opts Example: MyStorage . build_url ( &quot;path/to&quot; , &quot;file.ext&quot; , some : :opt ) #=&gt; &quot;https://my-storage/my-bucket/path/to/file.ext&quot;","ref":"Trunk.Storage.html#c:build_uri/3","title":"Trunk.Storage.build_uri/3","type":"callback"},{"doc":"Deletes the version file from the storage system at path Path.join(directory, filename) directory - The directory filename - The name of the file opts - The options for the storage system, set in Trunk option :storage_opts Example: The file should be removed from &lt;storage system&gt;/my-bucket/path/to/file.ext MyStorage . delete ( &quot;path/to/&quot; , &quot;file.ext&quot; , some : :opt )","ref":"Trunk.Storage.html#c:delete/3","title":"Trunk.Storage.delete/3","type":"callback"},{"doc":"","ref":"Trunk.Storage.html#c:retrieve/4","title":"Trunk.Storage.retrieve/4","type":"callback"},{"doc":"Save the source_file to the storage system on path Path.join(directory, filename) directory - The directory filename - The name of the file source_path - The full path to the file to be stored. This is a path to the uploaded file or a temporary file that has undergone transformation opts - The options for the storage system, set in Trunk option :storage_opts Example: The file should be saved to &lt;storage system&gt;/my-bucket/path/to/file.ext MyStorage . save ( &quot;path/to/&quot; , &quot;file.ext&quot; , &quot;/tmp/uploaded_file.ext&quot; , some : :opt )","ref":"Trunk.Storage.html#c:save/4","title":"Trunk.Storage.save/4","type":"callback"},{"doc":"","ref":"Trunk.Storage.html#t:opts/0","title":"Trunk.Storage.opts/0","type":"type"},{"doc":"A Trunk.Storage implementation for the local file system.","ref":"Trunk.Storage.Filesystem.html","title":"Trunk.Storage.Filesystem","type":"module"},{"doc":"Generates a URL for the storage directory and file directory - The relative directory where the file is saved filename - The name of the file that was previous saved opts - The options for the storage system base_url: (optional) The base url to join the path to Example: Trunk.Storage.Filesystem . build_url ( &quot;path/to&quot; , &quot;file.ext&quot; ) #=&gt; &quot;path/to/file.ext&quot; Trunk.Storage.Filesystem . build_url ( &quot;path/to&quot; , &quot;file.ext&quot; , base_uri : &quot;http://example.com&quot; ) #=&gt; &quot;http://example.com/path/to/file.ext&quot; Trunk.Storage.Filesystem . build_url ( &quot;path/to&quot; , &quot;file.ext&quot; , base_uri : &quot;http://example.com/uploads/&quot; ) #=&gt; &quot;http://example.com/uploads/path/to/file.ext&quot; Trunk.Storage.Filesystem . build_url ( &quot;path/to&quot; , &quot;file.ext&quot; , base_uri : &quot;/uploads/&quot; ) #=&gt; &quot;/uploads/path/to/file.ext&quot;","ref":"Trunk.Storage.Filesystem.html#build_uri/3","title":"Trunk.Storage.Filesystem.build_uri/3","type":"function"},{"doc":"Copies the file within the file system. directory - The relative directory within which to find the file filename - The name of the file to be copied from to_directory - The relative directory within which to copy the file to_filename - The name of the file to be copied to opts - The options for the storage system path: (required) The base path within which to save files acl: (optional) The file mode to store the file (accepts octal 0o644 or string &quot;0644&quot; ). See File.chmod/2 for more info. Example: The file will be copied from /opts/uploads/path/to/file.ext to /opts/uploads/copied/to/file.copy Trunk.Storage.Filesystem . copy ( &quot;path/to/&quot; , &quot;file.ext&quot; , &quot;copied/to/&quot; , &quot;file.copy&quot; , path : &quot;/opt/uploads&quot; )","ref":"Trunk.Storage.Filesystem.html#copy/5","title":"Trunk.Storage.Filesystem.copy/5","type":"function"},{"doc":"Deletes the file from the file system. directory - The relative directory within which to store the file filename - The name of the file to be saved opts - The options for the storage system path: (required) The base path within which to save files Example: The file will be removed from /opts/uploads/path/to/file.ext Trunk.Storage.Filesystem . delete ( &quot;path/to/&quot; , &quot;file.ext&quot; , path : &quot;/opt/uploads&quot; )","ref":"Trunk.Storage.Filesystem.html#delete/3","title":"Trunk.Storage.Filesystem.delete/3","type":"function"},{"doc":"Callback implementation for Trunk.Storage.retrieve/4 .","ref":"Trunk.Storage.Filesystem.html#retrieve/4","title":"Trunk.Storage.Filesystem.retrieve/4","type":"function"},{"doc":"Saves the file to the file system. directory - The relative directory within which to store the file filename - The name of the file to be saved source_path - The full path to the file to be stored. This is a path to the uploaded file or a temporary file that has undergone transformation opts - The options for the storage system path: (required) The base path within which to save files acl: (optional) The file mode to store the file (accepts octal 0o644 or string &quot;0644&quot; ). See File.chmod/2 for more info. Example: The file will be saved to /opts/uploads/path/to/file.ext Trunk.Storage.Filesystem . save ( &quot;path/to/&quot; , &quot;file.ext&quot; , &quot;/tmp/uploaded_file.ext&quot; , path : &quot;/opt/uploads&quot; )","ref":"Trunk.Storage.Filesystem.html#save/4","title":"Trunk.Storage.Filesystem.save/4","type":"function"},{"doc":"A Trunk.Storage implementation for Amazon’s S3 service.","ref":"Trunk.Storage.S3.html","title":"Trunk.Storage.S3","type":"module"},{"doc":"Generates a URL to the S3 object directory - The directory (will be combined with the filename to form the S3 object key. filename - The name of the file (will be combined with the directory to form the S3 object key. opts - The options for the storage system bucket: (required) The S3 bucket in which to store the object. virtual_host: (optional) boolean indicator whether to generate a virtual host style URL or not. signed: (optional) boolean whether to sign the URL or not. ex_aws: (optional) override options for ex_aws Example: Trunk.Storage.S3 . build_url ( &quot;path/to&quot; , &quot;file.ext&quot; , bucket : &quot;my-bucket&quot; ) #=&gt; &quot;https://s3.amazonaws.com/my-bucket/path/to/file.ext&quot; Trunk.Storage.S3 . build_url ( &quot;path/to&quot; , &quot;file.ext&quot; , bucket : &quot;my-bucket&quot; , virtual_host : true ) #=&gt; &quot;https://my-bucket.s3.amazonaws.com/path/to/file.ext&quot; Trunk.Storage.S3 . build_url ( &quot;path/to&quot; , &quot;file.ext&quot; , bucket : &quot;my-bucket&quot; , signed : true ) #=&gt; &quot;https://s3.amazonaws.com/my-bucket/path/to/file.ext?X-Amz-Algorithm=AWS4-HMAC-SHA256&amp;…&quot;","ref":"Trunk.Storage.S3.html#build_uri/3","title":"Trunk.Storage.S3.build_uri/3","type":"function"},{"doc":"Copies the file within Amazon S3. directory - The relative directory within which to find the file (will be combined with the filename to form the source S3 object key). filename - The name of the file to be copied from (will be combined with the directory to form the source S3 object key). to_directory - The relative directory within which to copy the file (will be combined with the to_filename to form the destination S3 object key). to_filename - The name of the file to be copied to (will be combined with the to_directory to form the destination S3 object key). opts - The options for the storage system path: (required) The base path within which to save files acl: (optional) The file mode to store the file (accepts octal 0o644 or string &quot;0644&quot; ). See File.chmod/2 for more info. Example: The file will be copied from s3.amazonaws.com/my-bucket/path/to/file.ext to s3.amazonaws.com/my-bucket/copied/to/file.copy Trunk.Storage.S3 . copy ( &quot;path/to/&quot; , &quot;file.ext&quot; , &quot;copied/to/&quot; , &quot;file.copy&quot; , bucket : &quot;my-bucket&quot; )","ref":"Trunk.Storage.S3.html#copy/5","title":"Trunk.Storage.S3.copy/5","type":"function"},{"doc":"Deletes the file from Amazon S3. directory - The directory (will be combined with the filename to form the S3 object key. filename - The name of the file (will be combined with the directory to form the S3 object key. opts - The options for the storage system bucket: (required) The S3 bucket in which to store the object ex_aws: (optional) override options for ex_aws Example: The file will be removed from s3.amazonaws.com/my-bucket/path/to/file.ext Trunk.Storage.S3 . delete ( &quot;path/to/&quot; , &quot;file.ext&quot; , bucket : &quot;my-bucket&quot; )","ref":"Trunk.Storage.S3.html#delete/3","title":"Trunk.Storage.S3.delete/3","type":"function"},{"doc":"Callback implementation for Trunk.Storage.retrieve/4 .","ref":"Trunk.Storage.S3.html#retrieve/4","title":"Trunk.Storage.S3.retrieve/4","type":"function"},{"doc":"Saves the file to Amazon S3. directory - The directory (will be combined with the filename to form the S3 object key). filename - The name of the file (will be combined with the directory to form the S3 object key). source_path - The full path to the file to be stored. This is a path to the uploaded file or a temporary file that has undergone transformation opts - The options for the storage system bucket: (required) The S3 bucket in which to store the object ex_aws: (optional) override options for ex_aws All other options are passed through to S3 put object which means you can pass in anything accepted by ExAws.S3.put_object_opts/0 including but not limited to :acl , :meta , :content_type , and :content_disposition Example: The file will be saved to s3.amazonaws.com/my-bucket/path/to/file.ext Trunk.Storage.S3 . save ( &quot;path/to/&quot; , &quot;file.ext&quot; , &quot;/tmp/uploaded_file.ext&quot; , bucket : &quot;my-bucket&quot; )","ref":"Trunk.Storage.S3.html#save/4","title":"Trunk.Storage.S3.save/4","type":"function"},{"doc":"This module defines a Trunk.VersionState struct and provides some helper functions for working with that state. Most of these fields are used internally during processing. Fields The following fields are available in the version state object. Some values are filled in during processing. temp_path - The path to the temporary file created for transformation. If the version doesn't undergo transformation, no temporary path will be available. transform - The transform instruction returned from Trunk.transform/2 storage_dir - The storage directory returned from Trunk.storage_dir/2 filename - The filename returned from Trunk.filename/2 storage_opts - The additional storage options returned from Trunk.storage_opts/2 assigns - shared user data as a map (Same as assigns in Plug.Conn ) Usage This information is made available during Trunk.postprocess/3 which is called once the transformation is complete but before the storage callbacks are called. At this point you can work with the transformed version file and assign data that can be used later when determining the storage directory, filename and storage options.","ref":"Trunk.VersionState.html","title":"Trunk.VersionState","type":"module"},{"doc":"Assigns a value to a key on the state. Example: iex&gt; version_state . assigns [ :hello ] nil iex&gt; version_state = assign ( version_state , :hello , :world ) iex&gt; version_state . assigns [ :hello ] :world","ref":"Trunk.VersionState.html#assign/3","title":"Trunk.VersionState.assign/3","type":"function"},{"doc":"","ref":"Trunk.VersionState.html#t:t/0","title":"Trunk.VersionState.t/0","type":"type"},{"doc":"MIT License Copyright (c) 2017 Andrew Timberlake Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions: The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html","title":"License","type":"extras"},{"doc":"Trunk A file attachment/storage library for Elixir.","ref":"readme.html","title":"Overview","type":"extras"},{"doc":"Add :trunk to your list of dependencies in mix.exs : def deps do [ { :trunk , &quot;~&gt; 1.0&quot; } , # If you want to use Amazon S3, then add: { :ex_aws_s3 , &quot;~&gt; 2.0&quot; } , { :hackney , &quot;~&gt; 1.7&quot; } , { :poison , &quot;~&gt; 3.1&quot; } , { :sweet_xml , &quot;~&gt; 0.6&quot; } , ] end Trunk has only one hard dependency on Briefly to handle temporary file creation (and auto-destruction)","ref":"readme.html#installation","title":"Overview - Installation","type":"extras"},{"doc":"Trunk is a behaviour that is implemented in a module. use Trunk creates functions for storing, deleting, and generating urls for your files and their versions. It then implements callbacks in the simplest way possible. You can then override the callbacks you need to as you want to extend the behaviour. defmodule MyTrunk do use Trunk , versions : [ :original , :thumb ] # override callbacks as needed. end","ref":"readme.html#usage","title":"Overview - Usage","type":"extras"},{"doc":"Trunk has been designed to be highly configurable. It can be configured in stages with each level merging with the level before it. See the documentation for all config options. Global configuration config :trunk , storage : Trunk.Storage.Filesystem , storage_opts : [ path : &quot;/tmp&quot; ] App specific configuration for umbrella type configs config :my_app , :trunk , storage : Trunk.Storage.S3 , storage_opts : [ bucket : &quot;test-trunk&quot; ] in order for these options to be used, you need to pass the otp_app option when calling use Trunk as follows: defmodule MyTrunk do use Trunk , otp_app : :my_app end Module configuration defmodule MyTrunk do use Trunk , versions : [ :original , :trunk ] , storage : Trunk.Storage.Filesystem , storage_opts : [ path : &quot;/tmp&quot; ] end Function options Caution: If you override options during the storage call, you need to be sure to pass the same options to other calls MyTrunk . store ( &quot;/path/to/file.ext&quot; , storage : Trunk.Storage.S3 , storage_opts : [ bucket : &quot;test-trunk&quot; ] )","ref":"readme.html#configuration","title":"Overview - Configuration","type":"extras"},{"doc":"Storage is handled by a behaviour. Two storage handlers are included: Trunk.Storage.Filesystem , and Trunk.Storage.S3 Additional storage systems can be handled by creating a module that implements the Trunk.Storage behaviour. When storing files, Trunk runs the file through a transformation pipeline allowing you to generate different versions of a file. Each stage in the pipeline is handled via a callback allowing you to configure what transformations take place, where the version is stored and how it is named. Full information can be found in the documentation","ref":"readme.html#storage","title":"Overview - Storage","type":"extras"},{"doc":"You have the option of passing a scope (usually a struct or map) into the transform functions. This scope object will then be available in each callback allowing you to further customise the handling of each version. Example: defmodule MyTrunk do use Trunk , versions : [ :thumb ] def storage_dir ( % Trunk.State { scope : %{ id : model_id } } , :thumb ) do : &quot;my_models/ \#{ model_id } &quot; end MyTrunk . store ( &quot;/path/to/file.ext&quot; , %{ id : 42 } ) # will save to &lt;storage&gt;/my_models/42/&lt;filename&gt;","ref":"readme.html#scope","title":"Overview - Scope","type":"extras"},{"doc":"State about the file and the version transformations is kept in a Trunk.State struct which is passed to each callback. Each version also keeps track of its own state in a Trunk.VersionState struct which is also available through the Trunk.State.versions map. Transformation One of the key features of Trunk is the ability to take a file and produce transformed versions. Perhaps you want to take an uploaded photo and produce a thumbnail, take a video and extract thumbnails, or take an XLS file and produce a CSV file for easier processing. This is all handled easily with the flexible version and transform system. See full documentation on Trunk.transform/2","ref":"readme.html#state","title":"Overview - State","type":"extras"},{"doc":"A shout out to stavro who created arc which I used in many of my projects, and which provided much inspiration for what resulted in Trunk. Photos used in testing coffee.jpg by Ronaldo Arthur Vidal coffee beans.jpg by Alex Jones","ref":"readme.html#credits","title":"Overview - Credits","type":"extras"},{"doc":"Copyright (c) 2017 Andrew Timberlake This work is free. You can redistribute it and/or modify it under the terms of the MIT License. See the LICENSE.md file for more details.","ref":"readme.html#copyright-and-license","title":"Overview - Copyright and License","type":"extras"},{"doc":"Examples","ref":"usage.html","title":"Usage","type":"extras"},{"doc":"This example does nothing but implement Trunk and override the Trunk.transform/2 callback for the :thumbnail version defmodule MyTrunk do use Trunk , versions : [ :original , :thumbnail ] def transform ( _state , :thumbnail ) , do : { :convert , &quot;-strip -thumbnail 100x100&gt;&quot; } def transform ( state , version ) , do : super ( state , version ) end { :ok , _state } = MyTrunk . store ( &quot;/path/to/picture.jpg&quot; ) # Will store original in &lt;storage&gt;/picture.jpg # and thumbnail in &lt;storage&gt;/picture_thumbnail.jpg","ref":"usage.html#simple-example","title":"Usage - Simple Example","type":"extras"},{"doc":"This example simply shows a full implementation with all the callback functions overridden. defmodule MyTrunk do use Trunk , versions : [ :original , :thumbnail ] , storage : Trunk.Storage.S3 , storage_opts : [ bucket : &quot;my-bucket&quot; , ex_aws : [ acl : :public_read ] ] def preprocess ( % Trunk.State { lower_extname : extname } = state ) do if extname in [ &quot;.png&quot; , &quot;.jpg&quot; , &quot;.jpeg&quot; ] do { :ok , state } else { :error , &quot;Invalid file&quot; } end end def transform ( _state , :thumbnail ) , do : { :convert , &quot;-strip -thumbnail 100x100&gt;&quot; , :jpg } def transform ( _state , :original ) , do : nil def postprocess ( % Trunk.VersionState { temp_path : temp_path } = version_state , :thumbnail , _state ) do hash = :crypto . hash ( :md5 , File . read! ( temp_path ) ) |&gt; Base . encode16 ( case : :lower ) { :ok , Trunk.VersionState . assign ( version_state , :hash , hash ) } end def postprocess ( version_state , version , state ) , do : super ( version_state , version , state ) def storage_opts ( _state , :original ) , do : [ acl : :private ] def storage_opts ( _state , :thumbnail ) , do : [ acl : :private , content_type : &quot;image/jpeg&quot; ] def storage_dir ( % Trunk.State { scope : % MyModel { id : model_id } } , _version ) , do : &quot;photos/ \#{ model_id } &quot; def filename ( state , :thumbnail ) , do : &quot;thumbnail_ \#{ Trunk.State . get_version_assign ( state , :thumbnail , :hash ) } .jpg&quot; def filename ( % Trunk.State { filename : filename } , :original ) , do : filename end iex&gt; my_model = % MyModel { id : 42 } iex&gt; { :ok , state } = MyTrunk . store ( &quot;/path/to/picture.png&quot; , my_model ) # Will store original in s3://my-bucket/photos/42/picture.png # and thumbnail in s3://my-bucket/photos/42/thumbnail_889c00fed0f5382b4bdea612ae7a42df.jpg iex&gt; %{ my_model | file : Trunk.State . save ( state , as : :map ) } % MyModel { id : 42 , file : %{ filename : &quot;picture.png&quot; , version_assigns : %{ thumbnail : %{ hash : &quot;889c00fed0f5382b4bdea612ae7a42df&quot; } } } } iex&gt; { :error , &quot;Invalid file&quot; } = MyTrunk . store ( &quot;/path/to/document.pdf&quot; , my_model )","ref":"usage.html#full-example","title":"Usage - Full Example","type":"extras"},{"doc":"Changelog","ref":"changelog.html","title":"Change Log","type":"extras"},{"doc":"Added CHANGELOG Add Trunk.copy/4 to copy files between &quot;folders&quot; within the same file system (different directores in filesystem or different object keys in same S3 bucket)","ref":"changelog.html#1-2-0","title":"Change Log - 1.2.0","type":"extras"},{"doc":"Return nil from Trunk.filename/2 to prevent storing a specific version","ref":"changelog.html#1-2-1","title":"Change Log - 1.2.1","type":"extras"},{"doc":"Returns nil from url/1,2,3,4 if the Trunk.filename/2 returns nil","ref":"changelog.html#1-3-0","title":"Change Log - 1.3.0","type":"extras"}]